require 'import'
import 'android.widget.Button'
import 'android.widget.LinearLayout'
import 'android.widget.TextView'
import 'android.app.Activity'
import 'android.view.View'

-- Oyun durumu (her açılışta sıfırlanır)
local board = {
    player = {4, 4, 4, 4, 4, 4, treasury = 0}, -- Oyuncu kuyuları ve hazinesi
    bot = {4, 4, 4, 4, 4, 4, treasury = 0}     -- Bot kuyuları ve hazinesi
}
local currentPlayer = "player"
local gameOver = false
local botLevel = nil
local gameStarted = false
local lastBotMove = "Henüz hamle yapılmadı" -- Rakibin son hamlesi
local gameResult = "" -- Oyun sonucu (kazanan)

-- Sesli bildirim
function announce(message)
    print(message) -- Jieshuo’da print sesli okunuyor
end

-- Tahta durumunu oku
function readBoard()
    local status = "Oyuncu kuyuları: "
    for i = 1, 6 do
        status = status .. "Kuyu " .. i .. ": " .. board.player[i] .. " taş, "
    end
    status = status .. "Hazine: " .. board.player.treasury .. " taş. "
    status = status .. "Bot kuyuları: "
    for i = 1, 6 do
        status = status .. "Kuyu " .. i .. ": " .. board.bot[i] .. " taş, "
    end
    status = status .. "Hazine: " .. board.bot.treasury .. " taş."
    announce(status)
end

-- Kurallar
function showRules()
    local rules = [[
    Türk Mangalası Kuralları:
    1. Her oyuncunun 6 kuyusu ve 1 hazinesi vardır. Her kuyuda 4 taş bulunur, toplam 48 taş.
    2. Oyuncu kendi kuyusundan taşları alır (2 veya daha fazla taş varsa 1 taş bırakır) ve saat yönünün tersine dağıtır. Kuyuda 1 taş varsa tüm taşlar alınır ve dağıtılır.
    3. Kendi hazinesine taş koyar, rakibin hazinesini atlar.
    4. Son taş rakibin kuyusuna düşer ve o kuyudaki taş sayısı çift olursa (2, 4, 6, ...), bu taşlar oyuncunun hazinesine gider. Kendi kuyusunda çift sayı oluşsa bile taşlar alınmaz.
    5. Son taş kendi kuyusuna düşer ve o kuyu 1 taş oluyorsa (öncesinde boşsa), karşısındaki rakip kuyusundaki taşlar ve kendi kuyusundaki 1 taş oyuncunun hazinesine gider.
    6. Son taş oyuncunun hazinesine düşerse, tekrar oynar.
    7. Bir oyuncunun kuyuları tamamen boşalırsa, rakip kalan taşları hazinesine alır. En çok taş toplayan kazanır.
    ]]
    announce(rules)
end

-- Hamle yapma
function makeMove(player, pit)
    local stones = player == "player" and board.player[pit] or board.bot[pit]
    if stones == 0 then
        announce("Bu kuyu boş, başka bir kuyu seçin.")
        return false
    end

    -- 1 taşlı kuyular için istisna: tüm taşlar alınır
    if stones == 1 then
        if player == "player" then
            board.player[pit] = 0
        else
            board.bot[pit] = 0
            lastBotMove = "Bot kuyu " .. pit .. "'yi seçti."
        end
    else
        -- 2 veya daha fazla taş varsa 1 taş bırak
        if player == "player" then
            board.player[pit] = 1
            stones = stones - 1
        else
            board.bot[pit] = 1
            stones = stones - 1
            lastBotMove = "Bot kuyu " .. pit .. "'yi seçti."
        end
    end

    local currentPit = pit
    local isPlayerSide = (player == "player")

    -- Taş dağıtımı
    while stones > 0 do
        currentPit = currentPit + 1

        if isPlayerSide then
            if currentPit <= 6 then
                board.player[currentPit] = board.player[currentPit] + 1
            elseif currentPit == 7 then
                board.player.treasury = board.player.treasury + 1
            else
                currentPit = currentPit - 7
                board.bot[currentPit] = board.bot[currentPit] + 1
                isPlayerSide = false
            end
        else
            if currentPit <= 6 then
                board.bot[currentPit] = board.bot[currentPit] + 1
            else
                currentPit = currentPit - 6
                board.player[currentPit] = board.player[currentPit] + 1
                isPlayerSide = true
            end
        end
        stones = stones - 1
    end

    -- Çift yapma kuralı (sadece rakibin kuyusu çift olunca alınır)
    if not isPlayerSide and currentPit <= 6 and board.bot[currentPit] % 2 == 0 and board.bot[currentPit] > 0 then
        local stolenStones = board.bot[currentPit]
        board.player.treasury = board.player.treasury + stolenStones
        board.bot[currentPit] = 0
        announce("Oyuncu kuyu çaldı! " .. stolenStones .. " taş alındı.")
    elseif isPlayerSide and currentPit <= 6 and board.player[currentPit] % 2 == 0 and board.player[currentPit] > 0 then
        announce("Kendi kuyunuzda çift sayı oluştu, taşlar alınmadı.")
    end

    -- Boş kuyuya taş atma kuralı (son taş kendi kuyusuna düşerse)
    if isPlayerSide and currentPit <= 6 and board.player[currentPit] == 1 then
        local oppositePit = 7 - currentPit
        if board.bot[oppositePit] > 0 then
            local stolenStones = board.bot[oppositePit]
            board.player.treasury = board.player.treasury + stolenStones + 1 -- Kendi kuyusundaki 1 taşı da al
            board.player[currentPit] = 0
            board.bot[oppositePit] = 0
            announce("Oyuncu boş kuyuya taş attı ve karşı kuyudan " .. stolenStones .. " taş aldı!")
        end
    elseif not isPlayerSide and currentPit <= 6 and board.bot[currentPit] == 1 then
        local oppositePit = 7 - currentPit
        if board.player[oppositePit] > 0 then
            local stolenStones = board.player[oppositePit]
            board.bot.treasury = board.bot.treasury + stolenStones + 1 -- Kendi kuyusundaki 1 taşı da al
            board.bot[currentPit] = 0
            board.player[oppositePit] = 0
            announce("Bot boş kuyuya taş attı ve karşı kuyudan " .. stolenStones .. " taş aldı!")
        end
    end

    -- Tekrar oynama
    if (isPlayerSide and currentPit == 7) or (not isPlayerSide and currentPit == 7) then
        announce(player == "player" and "Oyuncu tekrar oynuyor!" or "Bot tekrar oynuyor!")
        return true
    end

    -- Oyun sonu
    local playerEmpty = true
    local botEmpty = true
    for i = 1, 6 do
        if board.player[i] > 0 then playerEmpty = false end
        if board.bot[i] > 0 then botEmpty = false end
    end
    if playerEmpty or botEmpty then
        gameOver = true
        if playerEmpty then
            for i = 1, 6 do
                board.bot.treasury = board.bot.treasury + board.bot[i]
                board.bot[i] = 0
            end
        end
        if botEmpty then
            for i = 1, 6 do
                board.player.treasury = board.player.treasury + board.player[i]
                board.player[i] = 0
            end
        end
        announce("Oyun bitti! Oyuncu hazinesi: " .. board.player.treasury .. ", Bot hazinesi: " .. board.bot.treasury)
        if board.player.treasury > board.bot.treasury then
            gameResult = "Kazanan: Oyuncu"
            announce("Tebrikler dostum, kazandın!")
        elseif board.bot.treasury > board.player.treasury then
            gameResult = "Kazanan: Bot"
            announce("Maalesef bot kazandı.")
        else
            gameResult = "Berabere"
            announce("Berabere!")
        end
    end

    return false
end

-- Bot hamlesi
function botMove()
    local pit
    if botLevel == "easy" then
        repeat
            pit = math.random(1, 6)
        until board.bot[pit] >= 1
    elseif botLevel == "medium" then
        pit = nil
        for i = 1, 6 do
            if board.bot[i] >= 1 then
                local targetPit = i + (board.bot[i] - (board.bot[i] > 1 and 1 or 0))
                if targetPit == 7 or (targetPit <= 6 and board.bot[targetPit] % 2 == 0) or (targetPit <= 6 and board.bot[targetPit] == 0 and board.player[7 - targetPit] > 0) then
                    pit = i
                    break
                end
            end
        end
        if not pit then
            repeat
                pit = math.random(1, 6)
            until board.bot[pit] >= 1
        end
    else -- Zor
        local bestPit = 1
        local bestScore = -1
        for i = 1, 6 do
            if board.bot[i] >= 1 then
                local score = 0
                local targetPit = i + (board.bot[i] - (board.bot[i] > 1 and 1 or 0))
                if targetPit == 7 then score = score + 5 end
                if targetPit <= 6 and board.bot[targetPit] % 2 == 0 then
                    score = score + board.bot[targetPit]
                end
                if targetPit <= 6 and board.bot[targetPit] == 0 and board.player[7 - targetPit] > 0 then
                    score = score + board.player[7 - targetPit]
                end
                if score > bestScore then
                    bestScore = score
                    bestPit = i
                end
            end
        end
        pit = bestPit
    end

    lastBotMove = "Bot kuyu " .. pit .. "'yi seçti."
    announce(lastBotMove)
    return makeMove("bot", pit)
end

-- Oyunu sıfırlama
function resetGame()
    board.player = {4, 4, 4, 4, 4, 4, treasury = 0}
    board.bot = {4, 4, 4, 4, 4, 4, treasury = 0}
    currentPlayer = "player"
    gameOver = false
    gameStarted = false
    botLevel = nil
    lastBotMove = "Henüz hamle yapılmadı"
    gameResult = ""
    updateLayout()
    announce("Oyun sıfırlandı! Zorluk seviyesini seçin ve Başlat butonuna basın.")
end

-- Layout oluşturma
layout = LinearLayout(this)
layout.setOrientation(1)

-- Butonları ve metni dinamik olarak güncelle
function updateLayout()
    layout.removeAllViews()

    -- Rakip durumu ve oyun sonucu metni
    local textRakipDurum = TextView(this)
    local rakipKuyular = "Rakip kuyuları: "
    for i = 1, 6 do
        rakipKuyular = rakipKuyular .. "Kuyu " .. i .. ": " .. board.bot[i] .. " taş, "
    end
    textRakipDurum.Text = "Son rakip hamlesi: " .. lastBotMove .. "\n" .. rakipKuyular .. "\n" .. gameResult
    layout.addView(textRakipDurum)

    if not gameStarted then
        -- Zorluk seçimi butonları
        local butonKolay = Button(this)
        butonKolay.Text = "Kolay Seviye"
        butonKolay.onClick = function()
            botLevel = "easy"
            announce("Kolay seviye seçildi. Oyunu başlatmak için Başlat butonuna basın.")
        end
        layout.addView(butonKolay)

        local butonOrta = Button(this)
        butonOrta.Text = "Orta Seviye"
        butonOrta.onClick = function()
            botLevel = "medium"
            announce("Orta seviye seçildi. Oyunu başlatmak için Başlat butonuna basın.")
        end
        layout.addView(butonOrta)

        local butonZor = Button(this)
        butonZor.Text = "Zor Seviye"
        butonZor.onClick = function()
            botLevel = "hard"
            announce("Zor seviye seçildi. Oyunu başlatmak için Başlat butonuna basın.")
        end
        layout.addView(butonZor)

        -- Başlat butonu
        local butonBaslat = Button(this)
        butonBaslat.Text = "Başlat"
        butonBaslat.onClick = function()
            if not botLevel then
                announce("Lütfen önce bir zorluk seviyesi seçin.")
                return
            end
            gameStarted = true
            gameOver = false
            board.player = {4, 4, 4, 4, 4, 4, treasury = 0}
            board.bot = {4, 4, 4, 4, 4, 4, treasury = 0}
            currentPlayer = "player"
            lastBotMove = "Henüz hamle yapılmadı"
            gameResult = ""
            announce("Oyun başladı! Oyuncu sırası.")
            readBoard()
            updateLayout()
        end
        layout.addView(butonBaslat)
    end

    -- Kurallar butonu
    local butonKurallar = Button(this)
    butonKurallar.Text = "Kurallar"
    butonKurallar.onClick = showRules
    layout.addView(butonKurallar)

    -- Skor butonu
    local butonSkor = Button(this)
    butonSkor.Text = "Toplam Skor"
    butonSkor.onClick = function()
        announce("Oyuncu hazinesi: " .. board.player.treasury .. ", Bot hazinesi: " .. board.bot.treasury)
    end
    layout.addView(butonSkor)

    -- Sıfırla butonu
    local butonSifirla = Button(this)
    butonSifirla.Text = "Skoru Sıfırla"
    butonSifirla.onClick = function()
        board.player.treasury = 0
        board.bot.treasury = 0
        announce("Skor sıfırlandı.")
    end
    layout.addView(butonSifirla)

    -- Kuyu butonları
    if gameStarted and not gameOver then
        for i = 1, 6 do
            local butonKuyu = Button(this)
            butonKuyu.Text = "Kuyu " .. i .. " (" .. board.player[i] .. " taş)"
            butonKuyu.onClick = function()
                if currentPlayer == "player" and not gameOver then
                    if makeMove("player", i) then
                        readBoard()
                    else
                        readBoard()
                        currentPlayer = "bot"
                        while currentPlayer == "bot" and not gameOver do
                            if not botMove() then
                                currentPlayer = "player"
                            end
                            readBoard()
                        end
                    end
                    updateLayout()
                end
            end
            layout.addView(butonKuyu)
        end
    end

    -- Tekrar başlat butonu (oyun bittiğinde)
    if gameOver then
        local butonTekrarBaslat = Button(this)
        butonTekrarBaslat.Text = "Tekrar Başlat"
        butonTekrarBaslat.onClick = function()
            resetGame()
        end
        layout.addView(butonTekrarBaslat)
    end

    activity.setContentView(layout)
end

-- Oyun başlatma
math.randomseed(os.time())
resetGame()